#!/usr/bin/env python

import os.path
import numpy as np

# this is a generic LC normalization function
# https://github.com/waqasbhatti/astrobase/blob/58c86723311162be49aa3f0e0df0872153cdb02e/astrobase/lcmath.py#L142
from astrobase.lcmath import normalize_magseries

# a function that is used by the LC reader function
def read_objectcatalog(objectinfo_catalog):
    '''
    This just reads in the object catalog into a recarray.

    '''
    return np.genfromtxt(objectinfo_catalog,
                         names=True,
                         dtype='U20,f8,f8,f8,f8,f8,i8,U20,U20,U60',
                         delimiter='|',
                         autostrip=True)


# example of a LC reader function. any kwargs can be specified fully later when
# telling the LCC-Server about this function e.g., we might want to call this
# all the time with objectinfo_catalog='object-db.csv' for our example here
def read_lightcurve(lcfile,
                    objectinfo_catalog=None):
    '''This reads in a light curve into an lcdict.

    lcfile: path to the light curve file to read

    objectinfo_catalog: path to the object-db.csv file or the prepared record
                        array from that file.

    Returns an lcdict in the required format.

    '''

    # get the object catalog first
    if (isinstance(objectinfo_catalog, str) and
        os.path.exists(objectinfo_catalog)):

        objcat = read_objectcatalog(objectinfo_catalog)

    elif isinstance(objectinfo_catalog, np.ndarray):

        objcat = objectinfo_catalog

    else:

        print("no object info catalog provided, can't continue...")
        return

    # look up the filename that we're reading in
    objectinfo_ind = objcat['lcfilename'] == os.path.basename(lcfile)

    # prepare the objectinfo dict
    objectinfo_dict = {}
    for x in objcat.dtype.names:
        objectinfo_dict[x] = np.asscalar(objcat[x][objectinfo_ind])

    # read the light curve file
    lcdata = np.genfromtxt(
        lcfile,
        names=True,
        dtype='f8,i8,f8,f8,f8,f8,f8,f8,f8,f8,f8,f8,f8',
        delimiter=','
    )

    columns = lcdata.dtype.names

    # the initial lcdict
    lcdict = {
        'objectid':objectinfo_dict['objectid'],
        'objectinfo':objectinfo_dict,
        'columns':columns
    }

    # turn the columns into separate ndarrays
    for col in columns:
        lcdict[col] = lcdata[col]

    return lcdict


# example of a normalization function. any kwargs can be specified fully when
# telling the LCC-Server about this function e.g., we might want to call this
# all the time with timecol='rjd', magcol='aep_000' for our example here
def normalize_lcdict(lcdict, timecol=None, magcol=None):
    '''
    This normalizes the input lcdict using an astrobase function.

    timecol, magcol are the columns to use in this operation.

    You must write your changes back to the lcdict and return it.

    '''

    if timecol and magcol:
        normalized_times, normalized_mags = normalize_magseries(
            lcdict['rjd'],
            lcdict['aep_000']
        )
        if normalized_times is not None and normalized_mags is not None:
            lcdict[timecol] = normalized_times
            lcdict[magcol] = normalized_mags

    return lcdict
